#include <iostream>
#include "mraa.hpp"
#include "TinkerKit.hpp"

using namespace std;
using namespace mraa;

int main(void) {

	// Mostrar un mensaje de inicio
	cout << "Hello IOT2000." << endl;

	// Inicializar variables
	int pot_value;
	char buffer [4];		// La longitud del String no debera superar los 4 caracteres

	// (intentar) Inicializar la conexion UART
	Uart* serial;
	try {
		serial = new Uart(0);    // Initialise UART port 0 (pins 0 and 1 of the Arduino extension)
	} catch (exception& e) {
		cout << e.what() << endl;
		return ERROR_UNSPECIFIED;
	}

	// Ajustar el baud rate a 9600 bit/s
	if (serial->setBaudRate(9600) != SUCCESS) {
		cout << "Error setting baud rate" << endl;
		return ERROR_UNSPECIFIED;
	}

	// Ajustar el modo de paridad
	if (serial->setMode(8, UART_PARITY_NONE, 1) != SUCCESS) {
		cout << "Error setting parity" << endl;
		return ERROR_UNSPECIFIED;
	}

	// Ajustar el flowControl
	if (serial->setFlowcontrol(false, false) != SUCCESS) {
		cout << "Error setting flow control" << endl;
	}


	// Inicializar los pines de entrada y salida
	TKPotentiometer pot(I0);           // Potenciometro TinkerKit!
	TKLed led(O0);					   // Led TinkerKit!

	// Apagar el led en caso de estar encendido antes de la ejecucion del programa
	led.off();

	// Bucle del programa
	while (true) {
		pot_value = pot.read();         // Leer el valor ADC del potenciometro (de 0 a 1023)
		cout << "Potentiometer value: " << pot_value << endl; // Mostrar el valor leido
		sprintf(buffer, "%d", pot_value); // Almacenar el valor en buffer
		serial->write(buffer, 4);       // Enviar el valor por la conexion serie establecida
		if (led.state()) led.off();		// Led encendido, apagar
		else led.on();					// Led apagado, encender
		usleep(1000*1000);              // Esperar 1 segundo (10^6 ms) para la siguiente lectura
	}

	return 0;
}

